package it.uniroma1.dis.jasmine;

import java.util.StringTokenizer;

/**
 * questi sono oggetti associati biunivocamente ad ogni riga della sezione TRANS del file .smv;
 * nell'oggetto Wrapper sar presente un Vector che li mantiene tutti,e la costruzione di tali oggetti Nodo, il 
 * relativo settaggio delle variabili di istanza, e il salvataggio nel Wrapper potr avvenire a tempo di esecuzione
 * del visitor TreeTraductor;
 */
public class GraphNode {
	//le variabili di istanza qui sono dichiarate pubbliche cos che nel visitor si possano 
	//accedere liberamente senza bisogno di prevedere opportuni metodi get e set...
	
	//necessario: il valore del pc che bisogner scrivere nel file .smv nella parte
	//sinistra della riga del TRANS associata a tale Nodo 
	private int initPC;   
	
	//opzionale,presente nei casi degli Statement che prevedono una scelta condizionale
	private Condition condition;   
	
	//necessario,  il prossimo valore del PC,che va scritto a destra della riga di TRANS associata
	private int destPC;    
	
	//opzionale,presente solo nei casi in cui prevedo un assegnazione o un side-effect su 
	//un parametro formale di tipo intero, o su una variabile locale, e qui si memorizza la
	//relativa espressione;
	private String expression;
	
	//opzionale presente solo nei casi in cui prevedo un assegnazione o un side-effect su 
	//un parametro formale di tipo intero, o su una variabile locale qui si memorizza la
	//relativa variabile su cui viene fatto il side-effect.
	private String variable;
	
	// opzionale presente solo nei casi in cui l'assegnazione o la condizione viene effettuata 
	// mediante una chiamata ad un metodo all'interno della stessa classe.
	private MethodCall methodCall;
	
	//opzionale, serve per rendere piu' chiara l'etichetta di un nodo switch nel grafo disegnato
	private boolean isSwitch=false;
	
	/**
	 * @param iPC
	 * @param dPC
	 */
	public GraphNode(int iPC,int dPC){
		initPC=iPC;
		destPC=dPC;
	}

	/**
	 * @param iPC
	 * @param dPC
	 * @param cond
	 */
	public GraphNode(int iPC,int dPC,Condition cond){
		initPC=iPC;
		destPC=dPC;
		condition=spaziaTrattini(cond);
	}

	/**
	 * @param iPC
	 * @param dPC
	 * @param var
	 * @param expr
	 */
	public GraphNode(int iPC,int dPC,String var,String expr){
		initPC=iPC;
		destPC=dPC;
		expression=spaziaTrattini(expr);
		variable=var;
	}

	/**
	 * @return the initPC
	 */
	public int getInitPC() {
		return initPC;
	}

	/**
	 * @param initPC the initPC to set
	 */
	public void setInitPC(int initPC) {
		this.initPC = initPC;
	}

	/**
	 * @return the condition
	 */
	public Condition getCondition() {
		return condition;
	}

	/**
	 * @param condition the condition to set
	 */
	public void setCondition(Condition condition) {
		this.condition = spaziaTrattini(condition);
	}

	/**
	 * @return the destPC
	 */
	public int getDestPC() {
		return destPC;
	}

	/**
	 * @param destPC the destPC to set
	 */
	public void setDestPC(int destPC) {
		this.destPC = destPC;
	}

	/**
	 * @return the expression
	 */
	public String getExpression() {
		return expression;
	}

	/**
	 * @param expression the expression to set
	 */
	public void setExpression(String expression) {
		this.expression = spaziaTrattini(expression);
	}

	/**
	 * @return the methodCall
	 */
	public MethodCall getMethodCall() {
  		return methodCall;
	}

	/**
	 * @param methodCall the methodCall to set
	 */
	public void setMethodCall(MethodCall methodCall) {
		this.methodCall = methodCall;
	}

	/**
	 * @return the variable
	 */
	public String getVariable() {
		return variable;
	}

	/**
	 * @param variable the variable to set
	 */
	public void setVariable(String variable) {
		this.variable = variable;
	}    
	
	public void setSwitch(boolean b) {
		isSwitch=b;
	}
	/**
	 * @return
	 */
	public String getDescription() {
		StringBuilder sb = new StringBuilder();
		sb.append(getInitPC());
		sb.append(": ");
		
		if (getCondition() != null)
			sb.append(getCondition().getImage());
		
		if (getVariable() != null) {
			sb.append(getVariable());
			sb.append(getExpression());
		}
		
		if (getInitPC() == getDestPC())
			sb.append("return");
		
		return sb.toString();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (isSwitch) {
			StringTokenizer st = new StringTokenizer(condition.toString()," <>=!");
			if (st.hasMoreTokens()) return getInitPC()+": switch("+st.nextToken()+")";
			else return getInitPC()+": switch";
		}
		else return getDescription();
/*		String result;
		result = "////////////////////////////////////////\n"
			+ "pc_di_partenza: "+initPC +"\n"
			+ "condizione: "+condition +"\n"
			+ "pc_di_destinazione: "+destPC +"\n" 
			+ "Side-effect Expression: "+expression +"\n"
			+ "Side-effect Variable: "+variable +"\n"
			+ "Method call: "+methodCall +"\n"
			+ "////////////////////////////////////////\n\n\n";
		return result;
*/	}
	
	//aggiunto da Andrea De Angelis il 10/10/08
	//serve a sistemare il problema che nusmv non riconosce ad esempio "a-b" ma vuole "a - b"
	public Condition spaziaTrattini(Condition c){
		String buffer="";
		String s=c.getImage();
		for(int i=0;i<s.length();i++){
			if((""+s.charAt(i)).equals("-")){
				if((""+s.charAt(i+1)).equals("-")){
					i=i+1;
				}
				else{
				buffer=buffer + " - ";
				}
			}else{
				buffer=buffer + s.charAt(i);
			}
		}
		Condition con=new Condition(buffer);
		return con;
	}
	
	public String spaziaTrattini(String s){
		String buffer="";
		for(int i=0;i<s.length();i++){
			if((""+s.charAt(i)).equals("-")){
				buffer=buffer + " - ";
			}else{
				buffer=buffer + s.charAt(i);
			}
		}
		return buffer;
	}
}
