/**
 * Jasmine
 * 29/mag/08
 */
package it.uniroma1.dis.jasmine;

import it.uniroma1.dis.jasmine.ui.JasmineFrame;
import it.uniroma1.dis.jasmine.ui.utils.NumberField;
import it.uniroma1.dis.jasmine.visitor.TranslatorVisitor;

import java.io.File;
import java.io.FileInputStream;
import java.util.Hashtable;

import javax.swing.JTextArea;

import EDU.purdue.jtb.parser.JTBParser;
import EDU.purdue.jtb.syntaxtree.CompilationUnit;

/**
 *
 */
public class JavaParser implements IJavaParser {
	private JTBParser parser;
	private static boolean firstCall = true;
	
	/**
	 * 
	 */
	public JavaParser() {
		super();
		parser = null;
	}
	
	/* (non-Javadoc)
	 * @see it.uniroma1.dis.jasmine.IJavaParser#generateGraph(java.io.File)
	 */
	@SuppressWarnings("static-access")
	public GraphWrapper generateGraph(File javaFile) {
		if ((javaFile == null) || !javaFile.exists())
			return null;
		
		try {
			if (firstCall) {
				parser = new JTBParser(new FileInputStream(javaFile));
				firstCall = false;
			} else
				JTBParser.ReInit(new FileInputStream(javaFile));
			
			CompilationUnit cu = parser.CompilationUnit();
			GraphWrapper w = new GraphWrapper();
			w.setIndirizzoFileJava(javaFile);
			TranslatorVisitor v = new TranslatorVisitor(w);
			cu.accept(v);
			
			
			return w;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see it.uniroma1.dis.javatosmv.IJavaParser#generateGraph(java.io.File)
	 */
	public String printGraph(File javaFile) {
		return printGraph(generateGraph(javaFile));
	}

	/* (non-Javadoc)
	 * @see it.uniroma1.dis.jasmine.IJavaParser#printGraph(it.uniroma1.dis.jasmine.GraphWrapper)
	 */
	public String printGraph(GraphWrapper wrapper) {
		if (wrapper == null)
			return null;
		
		StringBuilder sb = new StringBuilder();
		sb.append(wrapper.getClassName());
		sb.append("\n");
		
		for (Graph g : wrapper.getGraphs()) {
			sb.append(g.getMethodName());
			sb.append("\n");
			for (GraphNode node : g.getNodes())
				sb.append(node.toString());
		}
		sb.append("\n");
		
		return sb.toString();
	}

	/* (non-Javadoc)
	 * @see it.uniroma1.dis.javatosmv.IJavaParser#generateSMVFile(java.io.File)
	 */
	public boolean generateSMVFile(File javaFile, int[] nodes, int optCammino, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect) {
		return generateSMVFile(generateGraph(javaFile), nodes, optCammino, hash, prePost, sideEffect);
	}

	/* (non-Javadoc)
	 * @see it.uniroma1.dis.jasmine.IJavaParser#generateSMVFile(it.uniroma1.dis.jasmine.GraphWrapper)
	 */
	public boolean generateSMVFile(GraphWrapper wrapper, int[] nodes, int optCammino, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect) {
		if (wrapper == null)
			return false;
		
		FileSmvWriter fsw = new FileSmvWriter();
		String nomeFileSmv = wrapper.getClassName() + "_";
		for (Graph g : wrapper.getGraphs())
			fsw.writeFileNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, optCammino, hash,prePost, sideEffect);
		
		return true;
	}
	
	public boolean generateSMVFileBase(File javaFile, int[] nodes, int lenBase, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect, JasmineFrame jframe) {
		return generateSMVFileBase(generateGraph(javaFile), nodes, lenBase, hash, prePost, sideEffect, jframe);
	}
	
	public boolean generateSMVFileBase(GraphWrapper wrapper, int[] nodes, int lenBase, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect, JasmineFrame jasmineFrame) {
		if (wrapper == null)
			return false;
		
		FileSmvWriterBase fsw = new FileSmvWriterBase();
		String nomeFileSmv = wrapper.getClassName() + "_";
		for (Graph g : wrapper.getGraphs())
			fsw.writeFileNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, lenBase, hash,prePost, sideEffect);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see it.uniroma1.dis.jasmine.IJavaParser#generateSMV(it.uniroma1.dis.jasmine.GraphWrapper)
	 */
	public String generateSMV(GraphWrapper wrapper, int[] nodes, int optCammino, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect, boolean currentMethodOnly) {
		if (wrapper == null)
			return null;
		
		FileSmvWriter fsw = new FileSmvWriter();
		String nomeFileSmv = wrapper.getClassName() + "_";
		StringBuilder sb = new StringBuilder();
		
		for (Graph g : wrapper.getGraphs()) {
			if (currentMethodOnly) {
				if (wrapper.getCurrentMethod().equalsIgnoreCase(g.getMethodName())) {
					sb.append(fsw.generateNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, optCammino, hash, prePost, sideEffect));
					break;
				}
			} else
				sb.append(fsw.generateNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, optCammino, hash, prePost, sideEffect));
		}
		
		return sb.toString();
	}
	
	public String generateSMVBase(GraphWrapper wrapper, int[] nodes, int lenBase, Hashtable<String,NumberField[]> hash, Hashtable<String,JTextArea> prePost, boolean sideEffect, boolean currentMethodOnly) {
		if (wrapper == null)
			return null;
		
		FileSmvWriterBase fsw = new FileSmvWriterBase();
		String nomeFileSmv = wrapper.getClassName() + "_";
		StringBuilder sb = new StringBuilder();
		
		for (Graph g : wrapper.getGraphs()) {
			if (currentMethodOnly) {
				if (wrapper.getCurrentMethod().equalsIgnoreCase(g.getMethodName())) {
					sb.append(fsw.generateNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, lenBase, hash, prePost, sideEffect));
					break;
				}
			} else
				sb.append(fsw.generateNuSMV(nomeFileSmv + g.getMethodName(), g, nodes, lenBase, hash, prePost, sideEffect));
		}
		
		return sb.toString();
	}
	
}
