/**
 * Jasmine
 * 07/lug/08
 */
package it.uniroma1.dis.jasmine;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * @author
 *
 */
public class MethodCall implements Cloneable {
	private String methodName;
	private String returnType;
	private List<String> declaredParameters;
	private List<String> passedParameters;
	private String preCondition;
	private String postCondition;
	
	/**
	 * @param methodName
	 * @param returnType
	 */
	public MethodCall(String methodName, String returnType) {
		super();
		this.methodName = methodName;
		this.returnType = returnType;
		this.declaredParameters = new LinkedList<String>();
		this.passedParameters = new LinkedList<String>();
	}
	/**
	 * @return the methodName
	 */
	public String getMethodName() {
		return methodName;
	}
	/**
	 * @param methodName the methodName to set
	 */
	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}
	/**
	 * @return the returnType
	 */
	public String getReturnType() {
		return returnType;
	}
	/**
	 * @param returnType the returnType to set
	 */
	public void setReturnType(String returnType) {
		this.returnType = returnType;
	}
	/**
	 * @return the declaredParameters
	 */
	public List<String> getDeclaredParameters() {
		return declaredParameters;
	}
	/**
	 * @param declaredParameters the declaredParameters to set
	 */
	public boolean addDeclaredParameter(String parameter) {
		if (!this.declaredParameters.contains(parameter))
			return this.declaredParameters.add(parameter);
		else
			return false;
	}
	/**
	 * @return the passedParameters
	 */
	public List<String> getPassedParameters() {
		return passedParameters;
	}
	/**
	 * @param passedParameters the passedParameters to set
	 */
	public void setPassedParameters(List<String> passedParameters) {
		this.passedParameters = passedParameters;
	}
	/**
	 * @return the preCondition
	 */
	public String getPreCondition() {
		return preCondition;
	}
	/**
	 * @param preCondition the preCondition to set
	 */
	public void setPreCondition(String preCondition) {
		this.preCondition = preCondition;
	}
	/**
	 * @return the postCondition
	 */
	public String getPostCondition() {
		return postCondition;
	}
	/**
	 * @param postCondition the postCondition to set
	 */
	public void setPostCondition(String postCondition) {
		this.postCondition = postCondition;
	}
	
	/**
	 * Genera la dichiarazione del metodo, in forma 
	 * <pre>
	 * returnType methodName(parameter list)
	 * </pre>
	 * @return
	 */
	public String getDeclaration() {
		StringBuilder sb = new StringBuilder();
		sb.append(getReturnType());
		sb.append(" ");
		sb.append(getMethodName());
		sb.append("(");
		
		for (int i = 0; i < getDeclaredParameters().size(); i++) {
			String dp = getDeclaredParameters().get(i);
			sb.append(dp);
			if (i < getDeclaredParameters().size() - 2)
				sb.append(", ");
		}
		sb.append(")");
		
		return sb.toString();
	}
	
	/**
	 * Crea una copia, mantenendo i soli attributi di tipo stringa.
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {
		MethodCall mc = new MethodCall(methodName, returnType);
		mc.preCondition = preCondition;
		mc.postCondition = postCondition;
		mc.declaredParameters = new LinkedList<String>(declaredParameters);
		Collections.copy(mc.declaredParameters, declaredParameters);
		
		return mc;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.getMethodName();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((methodName == null) ? 0 : methodName.hashCode());
		result = prime * result
				+ ((declaredParameters == null) ? 0 : declaredParameters.hashCode());
		result = prime * result
				+ ((returnType == null) ? 0 : returnType.hashCode());
		return result;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final MethodCall other = (MethodCall) obj;
		if (methodName == null) {
			if (other.methodName != null)
				return false;
		} else if (!methodName.equals(other.methodName))
			return false;
		if (declaredParameters == null) {
			if (other.declaredParameters != null)
				return false;
		} else if (!declaredParameters.equals(other.declaredParameters))
			return false;
		if (returnType == null) {
			if (other.returnType != null)
				return false;
		} else if (!returnType.equals(other.returnType))
			return false;
		return true;
	}
}
