package it.uniroma1.dis.jasmine.ui;

import it.uniroma1.dis.jasmine.GraphNode;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;

/**
 *
 */
public class GraphViewer extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 4961069816010012672L;
	
	private Vector<GraphNode> nodes;
	private String methodName;
	private JGraph graph;
	private GraphModel model;
	private ISelectGraph graphSelector;

	
	/**
	 * Associa ad una cella grafica tutti quei nodi che hanno lo stesso PC di partenza.
	 */
	private Map<Integer, List<GraphNode>> graphNodeMap;
	
	/**
	 * Associa ad una cella grafica il suo PC.
	 */
	private Map<Integer, DefaultGraphCell> cellMap;
	
	int processati = 1;
	int switchCount = 0;
	/**
	 * @param v
	 * @param methodName
	 */
	public GraphViewer(Vector<GraphNode> v, String methodName, ISelectGraph graphSelector) {
		super();
		setLayout(new BorderLayout());
		this.nodes = v;
		this.methodName = methodName;
		this.graphSelector = graphSelector;
		graphNodeMap = new HashMap<Integer, List<GraphNode>>();
		cellMap = new HashMap<Integer, DefaultGraphCell>();
		model = new DefaultGraphModel();
		buildGraph();
	}
	
	/**
	 * @return the methodName
	 */
	public String getMethodName() {
		return methodName;
	}
	
	/**
	 * @return
	 */
	public BufferedImage getImage() {
		if (graph != null)
			return graph.getImage(null, 5);
		else
			return null;
	}
	
	/**
	 * 
	 */
	private void buildGraph() {
		// Construct Model and Graph
		graph = new JGraph(model);
		int arrow = GraphConstants.ARROW_CLASSIC;
		
		// control-drag should clone selection
		graph.setCloneable(false);
		
		// disable edit:
		graph.setInvokesStopCellEditing(false);
		
		// when over a cell, jump to its default port
		graph.setJumpToDefaultPort(true);
		
		graph.addMouseListener(new GraphMouseListener(graph));
		
		List<DefaultGraphCell> list = createGraphFromList(arrow);
		
		if (list.size() > 0) {
			DefaultGraphCell[] cells2 = list.toArray(new DefaultGraphCell[0]);
			for (Object d : cells2)
				System.out.println(d.getClass().getName());
			
			DefaultGraphCell[] cells3 = new DefaultGraphCell[1];
			if ((cells2 != null) && (cells2.length> 0)) {
				cells3[0] = list.get(0);
//				System.out.println(cells3[0]);
			}
			
			graph.getGraphLayoutCache().insert(cells2);
		}
		
		// show:
		this.add(new JScrollPane(graph));
	}
	
	/**
	 * @param arrow
	 * @return
	 */
	private List<DefaultGraphCell> createGraphFromList(int arrow) {
		LinkedList<DefaultGraphCell> list = new LinkedList<DefaultGraphCell>();
		if (nodes != null) {
			HashSet<Integer> pcSet = new HashSet<Integer>();
			for (GraphNode node : nodes) {
				pcSet.add(new Integer(node.getInitPC()));
			}
			
			// build cells:
			System.out.println("nodes: " + nodes.size() + " termPC: " + pcSet.size());
			int nodeCount = 0;
			int returnCount = 0;
			
			for (Integer pc : pcSet) {
				List<GraphNode> pcList = findByInitPC(pc, nodes);
				if (!pcList.isEmpty()) {
					int x = 50 + (nodeCount % 2) * 60;
					int y = 10 + nodeCount * 75;
					nodeCount++;
					
					GraphNode firstNode = pcList.get(0);
					Color color;
					if (firstNode.getCondition() != null)
						color = Color.CYAN;
					else if (firstNode.getVariable() != null)
						color = Color.ORANGE;
					else {
						color = Color.GREEN;
						returnCount++;
						x = 50 + returnCount * 75;
					}
					
					DefaultGraphCell cell = createVertex(firstNode, x, y, 100, 30, color, false);
					
					// add other ports for the eventual children:
					if (pcList.size() > 1)
						for (int j = 0; j < pcList.size(); j++)
							cell.addPort();
					
					list.add(cell);
					graphNodeMap.put(pc, pcList);
					cellMap.put(pc, cell);
				}
			}
			// build edges:
			for (Integer pc : pcSet) {
				int j = 0;
				for (GraphNode node : graphNodeMap.get(pc)) {
					list.add(connectCells(cellMap.get(pc), cellMap.get(node.getDestPC()), j++, arrow));
				}
			}
		}
		
		return list;
	}
	
	/**
	 * @param data
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 * @param bg
	 * @param raised
	 * @return
	 */
	private DefaultGraphCell createVertex(Object data, double x, double y, double w, double h, Color bg, boolean raised) {
		// Create vertex with the given name
		DefaultGraphCell cell = new DefaultGraphCell(data);
		
		// Set bounds
		GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(x, y, w, h));
		
		// Set fill color
		if (bg != null) {
			GraphConstants.setGradientColor(cell.getAttributes(), bg);
			GraphConstants.setOpaque(cell.getAttributes(), true);
		}
		
		// Set raised border
		if (raised)
			GraphConstants.setBorder(cell.getAttributes(), BorderFactory.createRaisedBevelBorder());
		else
			// Set black border
			GraphConstants.setBorderColor(cell.getAttributes(), Color.black);
		
		// Add a Floating Port
		cell.addPort();
		return cell;
	}
	
	/**
	 * @param src
	 * @param dest
	 * @param srcChildIndex
	 * @param arrow
	 * @return
	 */
	private DefaultEdge connectCells(DefaultGraphCell src, DefaultGraphCell dest, int srcChildIndex, int arrow) {
		DefaultEdge edge = new DefaultEdge();
		
		System.out.println("Countdown : "+(graphNodeMap.size()-processati));
		if (graphNodeMap.get(processati)!=null) {
			System.out.println("One");
			if (graphNodeMap.get(processati).get(switchCount)!=null) {
				System.out.println("Two");				
				if (graphNodeMap.get(processati).get(switchCount).getCondition()!=null)
					edge = new DefaultEdge(graphNodeMap.get(processati).get(switchCount).getCondition().toString());
			}
			if (switchCount<graphNodeMap.get(processati).size()-1)
				switchCount++;	
			else {
				processati++;	
				switchCount=0;
			}
		}
		if (src.getChildCount()>3) {
			System.out.println("trasformo in switch");
			GraphNode firstNode = (GraphNode)src.getUserObject();
			firstNode.setSwitch(true);
			src.setUserObject(firstNode);
		}
		/*if (src.getChildCount()>3) {
			edge = new DefaultEdge(graphNodeMap.get(2).get(1).getCondition().toString());
			//edge = new DefaultEdge(src.getUserObject().toString());	
		} 
		else if (src.getChildCount()>1) {
		if (srcChildIndex==0) edge = new DefaultEdge(src.getUserObject().toString());
		else if (srcChildIndex==1) edge = new DefaultEdge("!("+src.getUserObject().toString()+")");
		}*/
		// Fetch the ports from the new vertices, and connect them with the edge
		edge.setSource(src.getChildAt(srcChildIndex));
		edge.setTarget(dest.getChildAt(0));
		
		GraphConstants.setLineEnd(edge.getAttributes(), arrow);
		GraphConstants.setEndFill(edge.getAttributes(), true);
//		GraphConstants.setLineStyle(edge.getAttributes(), GraphConstants.STYLE_BEZIER);
//		GraphConstants.setRouting(edge.getAttributes(), GraphConstants.ROUTING_SIMPLE);
		GraphConstants.setBendable(edge.getAttributes(), true);
		GraphConstants.setDisconnectable(edge.getAttributes(), false);
		
		return edge;
	}
	
	/**
	 * @param pc
	 * @param nodeVector
	 * @return
	 */
	private List<GraphNode> findByInitPC(int pc, Vector<GraphNode> nodeVector) {
		LinkedList<GraphNode> list = new LinkedList<GraphNode>();
		
		for (GraphNode g : nodeVector)
			if (g.getInitPC() == pc)
				list.add(g);
		
		return list;
	}
	
	/**
	 *
	 */
	private class GraphMouseListener extends MouseAdapter {
		private JGraph graph;
		
		/**
		 * 
		 */
		public GraphMouseListener(JGraph graph) {
			super();
			this.graph = graph;
		}
		
		/* (non-Javadoc)
		 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
		 */
		@Override
		public void mouseClicked(MouseEvent me) {
			Object o = this.graph.getFirstCellForLocation(me.getX(), me.getY());
			if (o instanceof DefaultEdge) {
				DefaultEdge edge = (DefaultEdge)o;
//				System.out.println(edge.getSource());
//				System.out.println(edge.getTarget().getClass().getName());
			}
			
			if (me.getButton() != MouseEvent.BUTTON1) {
				if ((o instanceof DefaultGraphCell) && !(o instanceof DefaultEdge)) {
					GraphNode node = (GraphNode)((DefaultGraphCell)o).getUserObject();
					System.out.println("graphic method: " + node + " " + node.getMethodCall());
					if (node.getMethodCall() != null)
						graphSelector.selectGraph(node.getMethodCall().getMethodName());
				}
			}
		}
	}
}
