package it.uniroma1.dis.jasmine.ui;

import it.uniroma1.dis.jasmine.Graph;
import it.uniroma1.dis.jasmine.GraphWrapper;
import it.uniroma1.dis.jasmine.JavaParser;
import it.uniroma1.dis.jasmine.ui.utils.FileNameFilter;
import it.uniroma1.dis.jasmine.ui.utils.NuSMVVariableData;
import it.uniroma1.dis.jasmine.ui.utils.NumberField;
import it.uniroma1.dis.jasmine.ui.utils.Settings;
import it.uniroma1.dis.jasmine.ui.utils.TextComponentPopup;
import it.uniroma1.dis.jasmine.ui.utils.Settings.SettingsEnum;
import it.uniroma1.dis.jasmine.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jgraph.graph.DefaultEdge;

/**
 *
 */
public class JasmineFrame extends JFrame implements ISelectGraph , ActionListener, MouseListener{				// frame principale
	private static final long serialVersionUID = 1L;
	private JButton vediGrafoButton;
	private JButton smvButton;
	private JButton testButton;
	private JButton imageButton;
	private JButton nusmvButton;
	private JButton baseButton;
	private JMenuItem loadMenuItem;			
	private JMenu fileMenu; 
	private JMenu aboutMenu;
	private JMenu editMenu;
	private JMenuItem aboutMenuItem;			
	private JMenuItem editMenuItem;			
	private JPanel centerPanel;
	private JPanel highPanel;
	private JPanel lowPanel;
	private JTabbedPane graphTab;
	private JTabbedPane javaSmvTab;
//	private JTabbedPane smvTab;
	private JPanel codePanel;
	private JMenuBar menuBar;
	private JPanel buttonPanel;
	private JPanel graphPanel;
	private JTextArea avviso;
	private JTextArea javaText;
	private JTextArea smvText;
	private JScrollPane smvTextScroller;
	private File selectedFile = null;
	private GraphWrapper gw;
	private JFrame frame;
//	private SceltaCamminoFrame popup;
	private NumberField sceltaCammino;
	private JFileChooser chooser;
	private JFileChooser nusmvChooser;
	private JavaParser parser;
	private int[] cammino;
	private Hashtable<String,NumberField[]> hash;
	private Hashtable<String,JTextArea> prePost;
	private JComboBox testCombo;
	private boolean sideEffect=false;
	private HashMap<String, String> methodSMVmap;
	//serve a distinguere quando passo il controllo al frame nuSMV se vengo dalla scelta della base o da quella del cammino
	protected int ricercaBase;
	
	/**
	 * 
	 */
	public JasmineFrame() { 
		frame = this;
		this.setResizable(true);
		
		parser = new JavaParser();
		methodSMVmap = new HashMap<String, String>();
		GraphViewer g;
		final int DEFAULT_FRAME_WIDTH = 1000;
		final int DEFAULT_FRAME_HEIGHT = 700;
		setSize(DEFAULT_FRAME_WIDTH, DEFAULT_FRAME_HEIGHT);
		setIconImage(new ImageIcon(getClass().getResource("/Jasmine.jpg")).getImage());

		Dimension frameSize = this.getSize();
		setMinimumSize(frameSize);
		setLocation(
				(Toolkit.getDefaultToolkit().getScreenSize().width
						- frameSize.width) / 2,
						(Toolkit.getDefaultToolkit().getScreenSize().height
								- frameSize.height) /2);
		
		FileNameFilter javaFilter = new FileNameFilter("Java files", "java");
		FileNameFilter pngFilter = new FileNameFilter("PNG files", "png");
		
		chooser = new JFileChooser("filejava");
		chooser.addChoosableFileFilter(pngFilter);
		chooser.addChoosableFileFilter(javaFilter);
		if (Settings.getInstance().getSetting(SettingsEnum.lastJavaFile) != null)
			chooser.setSelectedFile(new File(Settings.getInstance().getSetting(SettingsEnum.lastJavaFile)));
		
		nusmvChooser = new JFileChooser();
		
		ActionListener listener = this;		// ACTION-LISTENER x pulsanti
		MenuListener mListener = new MenuListener();		// MENU LISTENER x menu
		TabChangeListener tabListener = new TabChangeListener();
		addWindowListener(new WindowCloser());
		/*
		 * barra dei menu
		 */
		menuBar = new JMenuBar();			
		setJMenuBar(menuBar);
		fileMenu = new JMenu("File");
		menuBar.add(fileMenu);
		loadMenuItem = new JMenuItem("Carica file java");
		loadMenuItem.addActionListener(mListener);
		fileMenu.add(loadMenuItem);
		editMenu = new JMenu("Modifica");
		menuBar.add(editMenu);
		editMenuItem = new JMenuItem("Impostazioni...");
		editMenuItem.addActionListener(mListener);
		editMenu.add(editMenuItem);
		aboutMenu = new JMenu("About");
		menuBar.add(aboutMenu);
		aboutMenuItem = new JMenuItem("About Jasmine");
		aboutMenuItem.addActionListener(mListener);
		aboutMenu.add(aboutMenuItem);
		/*
		 * pannello centrale
		 */
		codePanel = new JPanel();
		codePanel.setLayout(new GridLayout(1,2));
		JPanel javaPanel = new JPanel();
		javaPanel.setLayout(new BorderLayout());
		javaPanel.setToolTipText("Dettagli sul file Java");
		javaPanel.setBorder (new TitledBorder(new BevelBorder(BevelBorder.RAISED,Color.lightGray,Color.white,new Color(124, 124, 124),new Color(178, 178, 178)), " File java "));
//		javaText = new TextArea("", 14, 35, TextArea.SCROLLBARS_BOTH);
		javaText = new JTextArea(0, 20);
		javaText.setComponentPopupMenu(new TextComponentPopup(javaText));
		//	javaText.setFont(new java.awt.Font ("Helvetica", 0, 9));
		javaText.setEditable(false);
		//javaText.setVisible(false);
		JPanel smvPanel = new JPanel();
		smvPanel.setLayout(new BorderLayout());
		smvPanel.setToolTipText("Dettagli sul file SMV");
		smvPanel.setBorder (new TitledBorder(new BevelBorder(BevelBorder.RAISED,Color.lightGray,Color.white,new Color(124, 124, 124),new Color(178, 178, 178)), " Codice SMV "));
//		smvText = new TextArea("", 14, 35, TextArea.SCROLLBARS_BOTH);
		smvText = new JTextArea(0, 20);
		smvText.setEditable(false);
		smvText.setComponentPopupMenu(new TextComponentPopup(smvText));
		smvTextScroller=new JScrollPane(smvText);
		javaPanel.add(new JScrollPane(javaText), BorderLayout.CENTER);
		smvPanel.add(smvTextScroller, BorderLayout.CENTER);
		//codePanel.add(javaPanel);
		//codePanel.add(smvPanel);
		javaSmvTab=new JTabbedPane();
		javaSmvTab.addTab("java", javaPanel);
		javaSmvTab.addTab("SMV", smvPanel);
		
		codePanel.add(javaSmvTab);
		javaSmvTab.setPreferredSize(new Dimension(300,700));
		graphPanel = new JPanel();
		graphPanel.setLayout(new BorderLayout());
		
		graphTab = new JTabbedPane();
		graphTab.addChangeListener(tabListener);
		graphTab.setToolTipText("Visualizza il grafo di flusso del programma Java");
		graphTab.setBorder (new TitledBorder(new BevelBorder(BevelBorder.RAISED,Color.lightGray,Color.white,new Color(124, 124, 124),new Color(178, 178, 178)), " Grafo di flusso "));
		JPanel camminoPanel = new JPanel();
		camminoPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED,Color.lightGray,Color.white,new Color(124, 124, 124),new Color(178, 178, 178)), "Dimensione della base / Scelta cammino di test"));
		camminoPanel.setLayout(new BorderLayout());
		sceltaCammino = new NumberField();
		sceltaCammino.setComponentPopupMenu(new TextComponentPopup(sceltaCammino));
		JLabel label1=new JLabel("Inserisci la dimensione della base cercata oppure la lista ordinata di nodi separati da spazi e scegli il tipo di cammino da percorrere");
		camminoPanel.add(label1, BorderLayout.NORTH);
		camminoPanel.add(sceltaCammino, BorderLayout.CENTER);
		testCombo = new JComboBox();
		testCombo.addItem("Attraversamento");
		testCombo.addItem("Cammino ordinato");
		testCombo.addItem("Cammino completo");
		camminoPanel.add(testCombo, BorderLayout.EAST);
		graphPanel.add(graphTab, BorderLayout.CENTER);
		graphPanel.add(camminoPanel, BorderLayout.SOUTH);
		centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(codePanel, BorderLayout.WEST);
		centerPanel.add(graphPanel, BorderLayout.CENTER);

		/*
		 * pannello alto
		 */
		highPanel = new JPanel();			
		avviso = new JTextArea();
		avviso.setEditable(false);
		//avviso.setBackground(Color.green);
		avviso.setBackground(highPanel.getBackground());
		avviso.setFocusable(false);
		avviso.setText("Carica un file Java dal menu a tendina 'File', quindi clicca su 'Crea grafo' per vedere il relativo grafo.\n"+
			"Clicca su 'Ricerca Base' per ottenere una base di cammini. Lancia NuSmv e attendi il risultato. \nNel grafo dei cammini clicca su un arco che appartiene ad un cammino da testare e lancia NuSmv.");
		highPanel.add(avviso);

		/*
		 * pannello basso
		 */
		buttonPanel = new JPanel();			
		buttonPanel.setBorder (new EtchedBorder());
		buttonPanel.setBackground(new Color (153,153,204));
		vediGrafoButton = new JButton("Crea Grafo");
		vediGrafoButton.setToolTipText("Mostra il Grafo Di Flusso del file Java caricato");
		vediGrafoButton.addActionListener(listener);
		buttonPanel.add(vediGrafoButton);
		smvButton = new JButton("Cambia opzioni");
		smvButton.setToolTipText("Visualizza il corrispondente codice SMV");
		smvButton.addActionListener(listener);
		buttonPanel.add(smvButton);
		baseButton=new JButton("Ricerca Base");
		baseButton.setToolTipText("Genera il file nuSMV per la ricerca di una base di cammini");
		baseButton.addActionListener(listener);
		buttonPanel.add(baseButton);
		testButton = new JButton("Testa un cammino");
		testButton.setToolTipText("Scegli un cammino nel grafo di flusso, nuSMV generera' un caso di test appropriato");
		testButton.addActionListener(listener);
		buttonPanel.add(testButton);
		imageButton = new JButton("Esporta PNG");
		imageButton.setToolTipText("Esporta il grafo generato in un'immagine PNG");
		imageButton.addActionListener(listener);
		buttonPanel.add(imageButton);
		nusmvButton = new JButton("Esegui NuSMV");
		nusmvButton.setToolTipText("Esegui NuSMV con il codice SMV generato");
		nusmvButton.addActionListener(listener);
		buttonPanel.add(nusmvButton);
		lowPanel = new JPanel();			
		lowPanel.setBorder (new EtchedBorder());
		lowPanel.setBackground(new Color (153,153,204));
		lowPanel.add(buttonPanel);
		
		
		/*
		 * dispongo pannelli su frame
		 */
		Container contentPane = getContentPane();		
		contentPane.add(highPanel, BorderLayout.NORTH);
		contentPane.add(centerPanel, BorderLayout.CENTER);
		contentPane.add(lowPanel, BorderLayout.SOUTH);
	}
	
	/* (non-Javadoc)
	 * @see it.uniroma1.dis.jasmine.ui.ISelectGraph#selectGraph(java.lang.String)
	 */
	public void selectGraph(String methodName) {
		if (graphTab == null)
			return;
		
		for (int i = 0; i < graphTab.getTabCount(); i++) {
			GraphViewer viewer = (GraphViewer)graphTab.getComponentAt(i);
			System.out.println(i + " -> " + methodName + " " + viewer.getMethodName());
			if ((viewer != null) && methodName.equalsIgnoreCase(viewer.getMethodName())) {
				graphTab.setSelectedIndex(i);
				smvText.setText(methodSMVmap.get(viewer.getMethodName()));
				break;
			}
		}
	}
	
	public void actionPerformed(ActionEvent event){  //gestisce i pulsanti
		try {
			Object source = event.getSource();		// trova pulsante premuto

			if (source == baseButton){
				if (selectedFile==null) {
					JOptionPane.showMessageDialog(null, "Selezionare prima il file java da graficare");
					return;
				}
				//sto cercando una base
				
				
				
				if (sceltaCammino.getText().equals("")) {
					JOptionPane.showMessageDialog(null, "Inserire la dimensione voluta della base");
					return;
				}
				//int lenBase=Integer.parseInt(sceltaCammino.getText());
				ricercaBase=Integer.parseInt(sceltaCammino.getText());
				FileSmvWriterBase fsw=new FileSmvWriterBase();
				StringTokenizer sk = new StringTokenizer(sceltaCammino.getText(), " ");
				
				cammino = null;
				

//				for (String m : gw.getMethodList()) {
					gw.setCurrentMethod(((GraphViewer)graphTab.getSelectedComponent()).getMethodName());
//					methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMV(gw, null, opt, null, null, false, true));
					methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMVBase(gw, cammino, ricercaBase, RangeDialog.cloneHashtable(hash), prePost, sideEffect, true));
//				}
				smvText.setText(methodSMVmap.get(gw.getCurrentMethod()));
				JOptionPane.showMessageDialog(this, "Codice NuSmv per la ricerca di una base di dimensione "+ ricercaBase + " generato correttamente \n Clicca su 'Esegui NuSMV'", "Codice Generato", JOptionPane.INFORMATION_MESSAGE);
				
			}
			else if (source == vediGrafoButton)	{	// pulsante x vedere il grafo
				if (selectedFile==null) {
					JOptionPane.showMessageDialog(null, "Selezionare prima il file java da graficare");
					return;
				}
//				gw = parser.generateGraph(selectedFile);
//				parser.printGraph(gw);
				
				javaSmvTab.setSelectedIndex(1);
				
//				gw.setCurrentMethod(((GraphViewer)graphTab.getSelectedComponent()).getMethodName());
				//smvText.setText(parser.generateSMV(gw, null));
				List<String> methodList = gw.getMethodList();
				
				for (int i = 0; i < methodList.size(); i++) {
					String m = methodList.get(i);
					gw.setCurrentMethod(m);
					methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMV(gw, null, 0, null, null, false, true));
				}
				gw.selectFirstMethod();
				smvText.setText(methodSMVmap.get(gw.getCurrentMethod()));
				//cerca l'istruzione per riportare lo scroll in alto
				
				
				centerPanel.invalidate();
				graphTab.removeAll();
				int firstMethodIndex = 0;
				int i = 0;
				for (Graph g : gw.getGraphs()) {
					//crea le linguette con il nome del metodo
					graphTab.add(g.getMethodName(), new GraphViewer(g.getNodes(), g.getMethodName(), (JasmineFrame)frame));
					System.out.println("**************** "+g.getMethodName());
					if (g.getMethodName().equals(gw.getCurrentMethod()))
						firstMethodIndex = i;
					i++;
				}
				centerPanel.validate();
				graphTab.setSelectedIndex(firstMethodIndex);
			
			} else if (source == testButton)	{	// pulsante x generare il caso di test
/*				if(popup == null) {
					popup = new SceltaCamminoFrame();
					popup.okButton.addActionListener(new ButtonAction());
				}

				popup.setVisible(true);
			} else if( popup!=null && source == popup.okButton ) {
*/				
//				gw = parser.generateGraph(selectedFile);
//				StringTokenizer sk = new StringTokenizer(popup.camminoField.getText(), " ");
				StringTokenizer sk = new StringTokenizer(sceltaCammino.getText(), " ");
				if (sk.countTokens() <= 0) {
					JOptionPane.showMessageDialog(null, "Inserire i numeri associati ai nodi che fanno parte del cammino da testare", "Errore", JOptionPane.ERROR_MESSAGE);
					return;
				}
				//sto cercando un caso di test quindi non sto cercando una base
				ricercaBase=1;
				cammino = new int[sk.countTokens()];
				int count = 0;
				while(sk.hasMoreTokens()){
					cammino[count] = Integer.parseInt(sk.nextToken());
					System.out.println(cammino[count]);
					count++;
				}
				int opt = testCombo.getSelectedIndex()+1;

//				for (String m : gw.getMethodList()) {
					gw.setCurrentMethod(((GraphViewer)graphTab.getSelectedComponent()).getMethodName());
//					methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMV(gw, null, opt, null, null, false, true));
					methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMV(gw, cammino, opt, RangeDialog.cloneHashtable(hash), prePost, sideEffect, true));
//				}
				smvText.setText(methodSMVmap.get(gw.getCurrentMethod()));
				JOptionPane.showMessageDialog(this, "Codice NuSmv per la generazione del caso di test relativo al cammino selezionato generato correttamente \n Clicca su 'Esegui NuSMV'", "Codice Generato", JOptionPane.INFORMATION_MESSAGE);
				
			} else if (source == smvButton)	{	// pulsante x mostrare le opzioni
				if (selectedFile==null) {
					JOptionPane.showMessageDialog(null, "Selezionare prima il file java da graficare");
					return;
				}
//				gw = parser.generateGraph(selectedFile);
				
				
				Graph selectedGraph = null;
				GraphViewer viewer = (GraphViewer)graphTab.getSelectedComponent();
				for (Graph g:gw.getGraphs()) {
					if (g.getMethodName().equals(viewer.getMethodName())) {
						selectedGraph = g;
						break;
					}
				}
				
				hash = null;
				prePost = null;
				
				RangeDialog rf = new RangeDialog(frame, selectedGraph);
				rf.setVisible(true);
				hash = rf.evaluateRange();
				prePost = rf.evaluatePrePost();
				int opt = testCombo.getSelectedIndex()+1;
				
	/*			int option = JOptionPane.showConfirmDialog(frame, (Object)"Permettere il side effect di eventuali array in input?");
				
				if (option==JOptionPane.CANCEL_OPTION) {
					System.out.println("Ritorno");
					return;
				}
				else if (option==JOptionPane.YES_OPTION) {
					System.out.println("Yes");
					sideEffect=true;
				}
				else if (option==JOptionPane.NO_OPTION) {
					System.out.println("No");
					sideEffect=false;
				}*/
				
				// imposta pre e post condizioni del grafo corrente:
				gw.setCurrentMethod(viewer.getMethodName());
				gw.setCurrentPreCondition(prePost.get("pre").getText());
				gw.setCurrentPostCondition(prePost.get("post").getText());
				gw.setCurrentVariableHash(RangeDialog.cloneHashtable(hash));
				
				// genera e visualizza codice SMV del grafo corrente:
				methodSMVmap.put(gw.getCurrentMethod(), parser.generateSMV(gw, cammino, opt, RangeDialog.cloneHashtable(hash), prePost, sideEffect, true));
				smvText.setText(methodSMVmap.get(gw.getCurrentMethod()));
				
				if (hash==null) System.out.println("Hash e' null!!!");
				if (prePost==null) System.out.println("PrePost e' null!!!");
			
			} else if (source == imageButton) {		// pulsante per esportare in PNG
				if (gw == null) {
					JOptionPane.showMessageDialog(null, "Generare prima un grafo", "Informazione", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				
				GraphViewer viewer = (GraphViewer)graphTab.getSelectedComponent();
				File newFile = new File(gw.getClassName() + "_" + viewer.getMethodName() + ".png");
				chooser.setSelectedFile(newFile);
				if (chooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION)
					newFile = chooser.getSelectedFile();
				else
					return;
				
				if (newFile.exists()) {
					if (JOptionPane.showConfirmDialog(null, "Il file esiste, vuoi sovrascriverlo?", "File esistente", JOptionPane.WARNING_MESSAGE) != JOptionPane.OK_OPTION)
						return;
				}
				
				boolean saved = false;
				try {
					BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(newFile));
					BufferedImage img = viewer.getImage();
					if (img != null) {
						ImageIO.write(img, "png", out);
						out.close();
						saved = true;
					}
				} catch (Exception e) {
					saved = false;
					e.printStackTrace();
				}
				
				if (saved) {
					JOptionPane.showMessageDialog(null, "L'immagine e' stata salvata", "Informazione", JOptionPane.INFORMATION_MESSAGE);
				} else
					JOptionPane.showMessageDialog(null, "L'immagine non e' stata salvata", "Errore", JOptionPane.ERROR_MESSAGE);
			
			} else if (source == nusmvButton) {
				if ((graphTab == null) || (graphTab.getTabCount() <= 0)) {
					JOptionPane.showMessageDialog(null, "Generare prima un grafo", "Informazione", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				
				// chiedere conferma se le pre e post non sono state definite:
				boolean askConfirm = false;
				for (Graph g : gw.getGraphs())
					if (g.getMethodCall().getPreCondition() == null) {
						askConfirm = true;
						break;
					}
				
				if (askConfirm) {
					int response = JOptionPane.showConfirmDialog(null, "Per alcuni metodi non sono state definite pre e post condizioni.\r\nVerranno utilizzati valori di default. Continuare?", "Continuare?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
					if (response != JOptionPane.YES_OPTION)
						return;
				}
				
				GraphViewer viewer = (GraphViewer)graphTab.getSelectedComponent();
				String smvRelPath = "fileSMV" + File.separator + gw.getClassName() + "_" + viewer.getMethodName() + ".smv";
				
				// salva automaticamente su file, nella cartella predefinita:
				
				int opt = testCombo.getSelectedIndex()+1;
				
				if(ricercaBase>1)
					parser.generateSMVFileBase(gw, cammino, ricercaBase, RangeDialog.cloneHashtable(hash), prePost, sideEffect, this);
				else	
					parser.generateSMVFile(gw, cammino, opt, RangeDialog.cloneHashtable(hash), prePost, sideEffect);
				
				NuSMVFrame runner = new NuSMVFrame(smvRelPath, nusmvChooser,ricercaBase,parser,gw,cammino,RangeDialog.cloneHashtable(hash),prePost,sideEffect,this);
				runner.setVisible(true);
			}
		}catch (Exception ex) {
			ex.printStackTrace();
			JOptionPane.showMessageDialog(null, "Si e' verificato un errore durante l'operazione richiesta");
		}	
	}
	
	//grafica la base del metodo
	public void graficaBase(Vector<NuSMVVariableData> base){
		//seleziona prima su graphTab il metodo corrente
		int numeroMetodi=graphTab.getTabCount();
		//seleziona il GraphViewer del metodo corrente e ridisegnalo con la base
		for(int i=0;i<numeroMetodi;i++){
			GraphViewer grv=(GraphViewer)graphTab.getComponentAt(i);
			String m=gw.getCurrentMethod();
			if((grv).getMethodName().equals(m)){
				graphTab.setSelectedIndex(i);		
				grv=new GraphViewerBase(gw.currentGraph.getNodes(),m,(JasmineFrame)frame,base);
				graphTab.remove(i);
				graphTab.add(grv,m,i);
				graphTab.setSelectedIndex(i);
				
			}
		}
		//rigenera il grafico
		
	}
	
	
	/**
	 *
	 */
	private class MenuListener implements ActionListener	// intercetta eventi dei menu
	{ 
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent mEvent)	{ 
			Object mSource = mEvent.getSource();		// trova voce del menu scelta

			if (mSource == loadMenuItem) {			// menu CARICA FILE JAVA
				try
					{ 
					if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						javaSmvTab.setSelectedIndex(0);
						selectedFile = chooser.getSelectedFile();
						Settings.getInstance().setSetting(SettingsEnum.lastJavaFile, selectedFile.getAbsolutePath());
						Settings.getInstance().storeSettings();
						BufferedReader javaIn = null;
						javaIn = new BufferedReader(new FileReader(selectedFile));
						javaText.setText("");
						String scrivi = javaIn.readLine();
						while (scrivi!=null) {
							javaText.setText(javaText.getText()+scrivi+"\n");
							scrivi = javaIn.readLine();
						}
						
						javaIn.close();
						
						// reset degli altri campi:
						smvText.setText("");
						sceltaCammino.setText("");
						hash = null;
						cammino = null;
						prePost = null;
						sideEffect=false;
						graphTab.removeAll();
						
						// la creazione del wrapper avviene una sola volta, al caricamento del file:
						gw = parser.generateGraph(selectedFile);
					}
				} catch (IOException e) {
					JOptionPane.showMessageDialog(null, "Errore di apertura del file!");
				} catch (IllegalArgumentException e) {
					JOptionPane.showMessageDialog(null, "Il file non e' corretto.");
				}
			} else if (mSource == editMenuItem) {
				SettingsDialog sd = new SettingsDialog(frame);
				sd.setVisible(true);
			} else if (mSource == aboutMenuItem)			// pulsante ABOUT
				JOptionPane.showMessageDialog (null, "Jasmine\nJAva Symbolic Model INtErpreter\nVersione  " + JasmineUi.VERSION + "\n\n" +
						"Features:\n\n"+
						"- Creazione del grafo di flusso di una classe Java\n"+
						"- Gestione di chiamate a metodi\n"+
						"- Conversione di codice Java in codice nuSMV\n"+
						"- Individuazione di una base di cammini ricoprenti nel grafo\n"+
						"- Generazione di casi di test dato un cammino nel grafo di flusso\n\n"+
						"Autori: \n"+
						"    versione 2.0\n"+
						"        Andrea De Angelis <smaramba83@gmail.com>\n"+
						"    versione 1.0\n"+
						"        Luka Porrini <lukaporrini@gmail.com>" + '\n' + 
						"        Constantin Moldovanu <constantin.moldovanu@gmail.com>" + '\n' + 
						"        Emanuele Tatti <emanuele.tatti@gmail.com>" + '\n'); 
		}
		
	}

	/**
	 *
	 */
	
	/**
	 *
	 */
	private class TabChangeListener implements ChangeListener {
		/* (non-Javadoc)
		 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
		 */
		public void stateChanged(ChangeEvent ce) {
			if (ce.getSource() == graphTab) {
				if (graphTab.getSelectedComponent() != null)
					smvText.setText(methodSMVmap.get(((GraphViewer)graphTab.getSelectedComponent()).getMethodName()));
			}
		}
	}
	
	public void setSceltaCammino(String s){
		sceltaCammino.setText(s);
	}
	
	public void setTestComboIndex(int i){
		testCombo.setSelectedIndex(i);
	}
	
	public void mouseClicked(MouseEvent me){
		
	}
	
	public void mousePressed(MouseEvent me){}
	public void mouseReleased(MouseEvent me){}
	public void mouseEntered(MouseEvent me){}
	public void mouseExited(MouseEvent me){}
	/**
	 *
	 */
	private static class WindowCloser extends WindowAdapter	{// chiude finestra
		/* (non-Javadoc)
		 * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
		 */
		public void windowClosing(WindowEvent event) {
			System.exit(0);
		}
	}
	
}
