/**
 * Jasmine
 * 14/giu/08
 */
package it.uniroma1.dis.jasmine.ui;

import it.uniroma1.dis.jasmine.FileSmvWriterBase;
import it.uniroma1.dis.jasmine.GraphWrapper;
import it.uniroma1.dis.jasmine.JavaParser;
import it.uniroma1.dis.jasmine.ui.utils.NuSMVOutputParser;
import it.uniroma1.dis.jasmine.ui.utils.NuSMVOutputParserBase;
import it.uniroma1.dis.jasmine.ui.utils.NumberField;
import it.uniroma1.dis.jasmine.ui.utils.ProcessRunner;
import it.uniroma1.dis.jasmine.ui.utils.Settings;
import it.uniroma1.dis.jasmine.ui.utils.TextComponentPopup;
import it.uniroma1.dis.jasmine.ui.utils.Settings.SettingsEnum;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Date;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.UndoManager;

/**
 *
 */
public class NuSMVFrame extends JFrame implements Observer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 2124193081009240644L;

	private JPanel nusmvPanel;
	private JPanel smvPanel;
	private JPanel outputPanel;

	private JTextField nusmvText;
	private JTextField smvFileText;
	private JTextArea smvArea;
	private JTextArea outputArea;
	private JTextArea errorArea;
	private JButton findBut;
	private JButton runBut;
	private JButton stopBut;
	private JButton undoButton;
	private JButton redoButton;
	private JButton saveButton;
	private JFileChooser chooser;
	
	private String nusmvParameter = " -bmc -bmc_length 100";
	private String smvRelativePath;
	private File nusmv;
	private ProcessRunner runner;
	private NuSMVFrame self;
	private UndoManager undoManager;
	private JavaParser parser;
	private GraphWrapper gw;
	int[] cammino;
	private Hashtable<String, NumberField[]> hash;
	private Hashtable<String, JTextArea> prePost;
	private boolean sideEffect;
	private JasmineFrame jasmineFrame;
	public long startTime;
	public long stopTime;
	
	private int ricercaBase;

	/**
	 * @param smvRelativePath path relativo del file SMV da testare
	 * @param chooser file chooser, passato come parametro per convenienza
	 * @param sideEffectPar 
	 * @param prePostPar 
	 * @param hashtablePar 
	 * @param camminoPar 
	 * @param gwPar 
	 * @param buttonAction 
	 * @param parser 
	 */
	public NuSMVFrame(String smvRelativePath, JFileChooser chooser, int ricercaBasePar, JavaParser parserPar, GraphWrapper gwPar, int[] camminoPar, Hashtable<String, NumberField[]> hashTablePar, Hashtable<String, JTextArea> prePostPar, boolean sideEffectPar, JasmineFrame jasmineFramePar) {
		super("NuSMV");
		this.smvRelativePath = smvRelativePath;
		this.chooser = chooser;
		this.self = this;
		ricercaBase=ricercaBasePar;
		parser=parserPar;
		jasmineFrame=jasmineFramePar;
		
		gw=gwPar;
		cammino=camminoPar;
		hash=hashTablePar;
		prePost=prePostPar;
		sideEffect=sideEffectPar;
		
		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);

		this.setResizable(true);
		final int DEFAULT_FRAME_WIDTH = 700;
		final int DEFAULT_FRAME_HEIGHT = 600;
		setSize(DEFAULT_FRAME_WIDTH, DEFAULT_FRAME_HEIGHT);
		Dimension frameSize = this.getSize();
		setMinimumSize(frameSize);
		setLocation(
				(Toolkit.getDefaultToolkit().getScreenSize().width
						- frameSize.width) / 2,
						(Toolkit.getDefaultToolkit().getScreenSize().height
								- frameSize.height) /2);

		ActionListener listener = new NuSMVActionListener();		// ACTION-LISTENER x pulsanti
		undoManager = new UndoManager();

		nusmvPanel = new JPanel();
		nusmvPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Scegli la locazione di NuSMV"));
		nusmvPanel.setLayout(new GridLayout(2, 1));
		nusmvText = new JTextField(30);
		nusmvText.setComponentPopupMenu(new TextComponentPopup(nusmvText));
		String nusmv = "/Applications/NuSMV/bin/NuSMV";
		
		if (Settings.getInstance().getSetting(SettingsEnum.nusmvLocation) != null)
			nusmv = Settings.getInstance().getSetting(SettingsEnum.nusmvLocation);
		if (chooser.getSelectedFile() != null)
			nusmv = chooser.getSelectedFile().getPath();
		if (Settings.getInstance().getSetting(SettingsEnum.nusmvParameters) != null)
				nusmvParameter = " " + Settings.getInstance().getSetting(SettingsEnum.nusmvParameters);
		if(ricercaBase>1)
			nusmvText.setText(nusmv);
		else
			nusmvText.setText(nusmv + nusmvParameter);
		findBut = new JButton("Trova NuSMV...");
		findBut.setToolTipText("Seleziona il programma NuSMV da utilizzare");
		findBut.addActionListener(listener);
		smvFileText = new JTextField(20);
		smvFileText.setEditable(false);
		smvFileText.setComponentPopupMenu(new TextComponentPopup(smvFileText));
		smvFileText.setText(smvRelativePath);
		runBut = new JButton("Esegui NuSMV");
		runBut.setToolTipText("Esegui NuSMV sul codice fornito");
		runBut.addActionListener(listener);
		stopBut = new JButton("Ferma NuSMV");
		stopBut.setToolTipText("Ferma l'esecuzione di NuSMV");
		stopBut.setEnabled(false);
		stopBut.addActionListener(listener);
		JPanel panUp = new JPanel();
		panUp.add(nusmvText);
		panUp.add(findBut);
		JPanel panDown = new JPanel();
		panDown.add(smvFileText);
		panDown.add(runBut);
		panDown.add(stopBut);
		nusmvPanel.add(panUp);
		nusmvPanel.add(panDown);

		smvPanel = new JPanel();
		smvPanel.setLayout(new BorderLayout());
		smvPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Codice SMV"));
		smvArea = new JTextArea(0, 20);
		smvArea.setComponentPopupMenu(new TextComponentPopup(smvArea));
		smvArea.getDocument().addUndoableEditListener(undoManager);
		JPanel buttonPanel = new JPanel();
		undoButton = new JButton("Undo");
		undoButton.addActionListener(listener);
		undoButton.setToolTipText("Annulla le modifiche");
		redoButton = new JButton("Redo");
		redoButton.addActionListener(listener);
		redoButton.setToolTipText("Ripeti le modifiche");
		saveButton = new JButton("Salva SMV");
		saveButton.setToolTipText("Se modifichi il file SMV, ricordati di salvarlo!");
		saveButton.addActionListener(listener);
		buttonPanel.add(undoButton);
		buttonPanel.add(redoButton);
		buttonPanel.add(saveButton);
		smvPanel.add(new JScrollPane(smvArea), BorderLayout.CENTER);
		smvPanel.add(buttonPanel, BorderLayout.SOUTH);

		outputPanel = new JPanel();
		outputPanel.setLayout(new BorderLayout());
		outputPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Esito invocazione NuSMV"));
		outputArea = new JTextArea(0, 20);
		outputArea.setEditable(false);
		outputArea.setComponentPopupMenu(new TextComponentPopup(outputArea));
		errorArea = new JTextArea(5, 20);
		errorArea.setComponentPopupMenu(new TextComponentPopup(errorArea));
		JScrollPane errorScroll = new JScrollPane(errorArea);
		errorArea.setEditable(false);
		errorScroll.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Errori NuSMV"));
		outputPanel.add(new JScrollPane(outputArea), BorderLayout.CENTER);
		outputPanel.add(errorScroll, BorderLayout.SOUTH);

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, smvPanel, outputPanel);
		splitPane.setDividerLocation(0.3);

		getContentPane().add(nusmvPanel, BorderLayout.NORTH);
		getContentPane().add(splitPane, BorderLayout.CENTER);

		loadSMVFile();
	}

	
	public JasmineFrame getJasmineFrame(){
		return jasmineFrame;
	}
	/**
	 * 
	 */
	private void loadSMVFile() {
		if (smvRelativePath == null)
			return;

		try {
			FileReader fr = new FileReader(System.getProperty("user.dir") + File.separator + smvRelativePath);
			BufferedReader br = new BufferedReader(fr);

			String line = null;
			do {
				line = br.readLine();
				if (line != null)
					smvArea.setText(smvArea.getText() + line + "\n");
			} while (line != null);

			br.close();
			fr.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return
	 */
	private boolean saveSMVFile() {
		try {
			FileWriter fw = new FileWriter(System.getProperty("user.dir") + File.separator + smvRelativePath);
			BufferedWriter bw = new BufferedWriter(fw);

			bw.write(smvArea.getText());

			bw.close();
			fw.close();
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Ascolta messaggi da parte del runner.
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable obs, Object param) {
		if (obs instanceof ProcessRunner) {
			runBut.setEnabled(true);
			stopBut.setEnabled(false);
			
			Date d =new Date();
			stopTime=d.getTime();

			if ((param != null) && (param instanceof Exception))
				JOptionPane.showMessageDialog(this, ((Exception)param).getMessage(), "Errore", JOptionPane.ERROR_MESSAGE);
			else {
				
				if(NuSMVOutputParser.checkSmvString(runner.getProcessOutput()))
				{
					if(ricercaBase>1){
						TestCaseDialogBase popupBase;
						popupBase = new TestCaseDialogBase(this, NuSMVOutputParserBase.parseTestCase(runner.getProcessOutput()));
						popupBase.setVisible(true);
					}
					else{
						TestCaseDialog popup;
						popup = new TestCaseDialog(this, NuSMVOutputParser.parseTestCase(runner.getProcessOutput()));
						popup.setVisible(true);
					}
				}
				else{ 
					if(ricercaBase>2){
						JOptionPane.showMessageDialog(this, "Nessuna base di dimensione " + ricercaBase + " trovata. Passo alla ricerca di una base di dimensione " + (ricercaBase-1) +"\n" +"Tempo di elaborazione: "+ ((stopTime-startTime)/1000) + "sec", "Attenzione", JOptionPane.INFORMATION_MESSAGE);
						startTime=d.getTime();
						ricercaBase=ricercaBase-1;
						//parser.generateSMVFileBase(gw, cammino, ricercaBase, RangeDialog.cloneHashtable(hash), prePost, sideEffect,jasmineFrame);
						jasmineFrame.ricercaBase=ricercaBase-1;
						FileSmvWriterBase fsw=new FileSmvWriterBase();
						smvArea.setText(fsw.generateNuSMV("RicercaBase", gw.currentGraph, cammino, ricercaBase, RangeDialog.cloneHashtable(hash), prePost, sideEffect));
						saveSMVFile();
						//lancia nusmv
						runBut.dispatchEvent(new ActionEvent(runBut,213241,"evento"));
						String nusmvPath = nusmvText.getText();
						if ((nusmvPath == null) || (nusmvPath.length() == 0))
							return;

						String fileName = System.getProperty("user.dir") + File.separator + smvRelativePath;
						// crea il runner:
						runner = new ProcessRunner(outputArea, errorArea,nusmvPath,fileName);
						// mette questo oggetto in ascolto per #update(java.util.Observable, java.lang.Object)
						runner.addObserver(self);
						
						new Thread(runner).start();
						
						runBut.setEnabled(false);
						stopBut.setEnabled(true);
						
					}else{
						JOptionPane.showMessageDialog(this, "Nessun caso di test trovato \n" + "Tempo di elaborazione " + ((stopTime-startTime)/1000) + "sec", "Attenzione", JOptionPane.INFORMATION_MESSAGE);
					}
				}
			
			}
		
		}
	}

	/**
	 *
	 */
	private class NuSMVActionListener implements ActionListener {
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent event) {
			Object source = event.getSource();		// trova pulsante premuto

			if (source == findBut)	{	
				if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
					nusmv = chooser.getSelectedFile();
					runBut.setEnabled(true);
				}

				if (nusmv == null)
					runBut.setEnabled(false);
				else
					nusmvText.setText(nusmv.getPath() + nusmvParameter);
			} else if (source == runBut) {
				String nusmvPath = nusmvText.getText();
				Date d =new Date();
			startTime=d.getTime();
				if ((nusmvPath == null) || (nusmvPath.length() == 0))
					return;
				//apri la finestra di info (AttesaNusmv)
				String fileName = System.getProperty("user.dir") + File.separator + smvRelativePath;
				// crea il runner:
				runner = new ProcessRunner(outputArea, errorArea,nusmvPath,fileName);
				// mette questo oggetto in ascolto per #update(java.util.Observable, java.lang.Object)
				runner.addObserver(self);
				
				new Thread(runner).start();
				
				runBut.setEnabled(false);
				stopBut.setEnabled(true);
			} else if (source == stopBut) {
				if (runner != null)
					runner.requestStop();

				runBut.setEnabled(true);
				stopBut.setEnabled(false);
			} else if (source == redoButton) {
				try {
					undoManager.redo();
				} catch (CannotRedoException e) {
					Toolkit.getDefaultToolkit().beep();
				}
			} else if (source == undoButton) {
				try {
					undoManager.undo();
				} catch (CannotRedoException e) {
					Toolkit.getDefaultToolkit().beep();
				}
			} else if (source == saveButton) {
				if (saveSMVFile())
					JOptionPane.showMessageDialog(null, "Il file e' stato salvato", "Informazione", JOptionPane.INFORMATION_MESSAGE);
				else
					JOptionPane.showMessageDialog(null, "Il file non e' stato salvato", "Errore", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
}
