/**
 * Jasmine
 * 29/mag/08
 */
package it.uniroma1.dis.jasmine.visitor;

import it.uniroma1.dis.jasmine.Condition;
import it.uniroma1.dis.jasmine.GraphNode;
import it.uniroma1.dis.jasmine.GraphWrapper;
import it.uniroma1.dis.jasmine.MethodCall;
import it.uniroma1.dis.jasmine.ParametroFormale;
import it.uniroma1.dis.jasmine.Variabile;

import java.util.LinkedList;
import java.util.Set;

import EDU.purdue.jtb.syntaxtree.ArgumentList;
import EDU.purdue.jtb.syntaxtree.Assignment;
import EDU.purdue.jtb.syntaxtree.BreakStatement;
import EDU.purdue.jtb.syntaxtree.DoStatement;
import EDU.purdue.jtb.syntaxtree.Expression;
import EDU.purdue.jtb.syntaxtree.ForInit;
import EDU.purdue.jtb.syntaxtree.ForStatement;
import EDU.purdue.jtb.syntaxtree.ForUpdate;
import EDU.purdue.jtb.syntaxtree.FormalParameter;
import EDU.purdue.jtb.syntaxtree.IfStatement;
import EDU.purdue.jtb.syntaxtree.LocalVariableDeclaration;
import EDU.purdue.jtb.syntaxtree.MethodDeclaration;
import EDU.purdue.jtb.syntaxtree.Name;
import EDU.purdue.jtb.syntaxtree.Node;
import EDU.purdue.jtb.syntaxtree.NodeChoice;
import EDU.purdue.jtb.syntaxtree.NodeListOptional;
import EDU.purdue.jtb.syntaxtree.NodeSequence;
import EDU.purdue.jtb.syntaxtree.PostfixExpression;
import EDU.purdue.jtb.syntaxtree.PreDecrementExpression;
import EDU.purdue.jtb.syntaxtree.PreIncrementExpression;
import EDU.purdue.jtb.syntaxtree.PrimitiveType;
import EDU.purdue.jtb.syntaxtree.ReturnStatement;
import EDU.purdue.jtb.syntaxtree.StatementExpression;
import EDU.purdue.jtb.syntaxtree.StatementExpressionList;
import EDU.purdue.jtb.syntaxtree.SwitchLabel;
import EDU.purdue.jtb.syntaxtree.SwitchStatement;
import EDU.purdue.jtb.syntaxtree.UnmodifiedClassDeclaration;
import EDU.purdue.jtb.syntaxtree.UnmodifiedInterfaceDeclaration;
import EDU.purdue.jtb.syntaxtree.WhileStatement;
import EDU.purdue.jtb.visitor.DepthFirstVisitor;

/**
 *
 */
public class TranslatorVisitorOLD extends DepthFirstVisitor {
	private GraphWrapper wrapper;
	private LinkedList<String> methodStack;
	private Set<MethodCall> methodSet;
	private boolean hasReturn;
	private int breakPoint;

	/**
	 * @param w
	 */
	public TranslatorVisitorOLD(GraphWrapper w) {
		wrapper = w;
		methodStack = new LinkedList<String>();
		hasReturn = false;
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.UnmodifiedClassDeclaration)
	 */
	@Override
	public void visit(UnmodifiedClassDeclaration n) {
		System.out.println("found class: " + n.f1.tokenImage);
		wrapper.setClassName(n.f1.tokenImage);

		MethodVisitor mv = new MethodVisitor();
		n.f4.accept(mv);
		this.methodSet = mv.getMethodSet();
		
		for (MethodCall m : methodSet)
			System.out.println("has method: " + m.getMethodName());

		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.UnmodifiedInterfaceDeclaration)
	 */
	@Override
	public void visit(UnmodifiedInterfaceDeclaration n) {
		System.out.println("found interface: " + n.f1.tokenImage);
		wrapper.setClassName(n.f1.tokenImage);

		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.MethodDeclaration)
	 */
	@Override
	public void visit(MethodDeclaration n) {
		// salva il metodo precedente:
		methodStack.addFirst(wrapper.getCurrentMethod());
		// imposta il metodo corrente:
		wrapper.setCurrentMethod(n.f2.f0.tokenImage);
		// associa il metodo al grafo:
		for (MethodCall mc : methodSet) {
			if (n.f2.f0.tokenImage.equals(mc.getMethodName())) {
				wrapper.setCurrentMethodCall(mc);
				break;
			}
		}
		// visita il metodo corrente:
		super.visit(n);

		System.out.println("hasReturn: " + hasReturn);
		// se non ha un return, aggiungerne uno "fittizio":
		if (!hasReturn) {
			wrapper.setLoc(wrapper.getLoc() + 1);
			GraphNode node = new GraphNode(wrapper.getLoc(), wrapper.getLoc());
			wrapper.addNode(node);
		}

		// imposta il metodo precedente:
		wrapper.setCurrentMethod(methodStack.removeFirst());
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.FormalParameter)
	 */
	@Override
	public void visit(FormalParameter n) {
		ParametroFormale pf= new ParametroFormale();
		pf.nome=n.f2.f0.tokenImage;
		System.out.println(n.f2.f0.tokenImage);
		if (n.f1.f1.present()|| n.f2.f1.present())
			pf.isArray=true; 

		if ((n.f1.f0.which)==0) {
			//siamo nel caso di Type() di tipo PrimitiveType(), e non di tipo Name(),per i tipi Ogetto che non sono trattati
			String t = typeVariable(((PrimitiveType)n.f1.f0.choice).f0);
			pf.tipo=t;
		}

		wrapper.getParameterSet().addElement(pf);

		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.LocalVariableDeclaration)
	 */
	@Override
	public void visit(LocalVariableDeclaration n) {
		wrapper.setLoc(wrapper.getLoc() + 1);

		Variabile v=new Variabile();
		v.setName(n.f2.f0.f0.tokenImage);
		System.out.println(n.f2.f0.f0.tokenImage);
		if (n.f1.f1.present()|| n.f2.f0.f1.present())
			v.setIsArray(true); 

		// tipo:
		if((n.f1.f0.which)==0){
			//siamo nel caso di Type() di tipo PrimitiveType(), e non di tipo Name(),per i tipi Ogetto non trattati
			String t=typeVariable(((PrimitiveType) n.f1.f0.choice).f0);
			v.setTipo(t);
		}
		wrapper.getVariableSet().addElement(v);

		ImageVisitor img = new ImageVisitor();
		n.f2.f1.accept(img);
		String assignment = img.getImage();
		GraphNode node = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1, v.getName(), assignment);
		wrapper.addNode(node);

		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.ArgumentList)
	 */
	@Override
	public void visit(ArgumentList n) {
		ImageVisitor img = new ImageVisitor();
		n.accept(img);
		
		// parsing dei parametri presenti in una chiamata a metodo:
		// primo parametro:
		img = new ImageVisitor();
		n.f0.accept(img);
		LinkedList<String> paramList = new LinkedList<String>();
		paramList.add(stripSpace(img.getImage()));
		
		// parametri successivi:
		for (Node opt : n.f1.nodes) {
			if (opt instanceof NodeSequence) {
				Node node = ((NodeSequence)opt).nodes.get(1);
				if (node instanceof Expression) {
					img = new ImageVisitor();
					node.accept(img);
					paramList.add(stripSpace(img.getImage()));
				}
			}
		}
		
		// prende il nodo corrente e gli imposta i parametri della chiamata a metodo:
		GraphNode prevNode = wrapper.getNodes().get(wrapper.getNodes().size() - 1);
		MethodCall prevMc = prevNode.getMethodCall();
		if (prevMc != null)
			prevMc.setPassedParameters(paramList);
		
		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.Name)
	 */
	@Override
	public void visit(Name n) {
		ImageVisitor img = new ImageVisitor();
		n.accept(img);
		System.out.println("Name: " + img.getImage());

		String m = stripSpace(img.getImage());
//		if (methodSet != null) {&& methodSet.contains(m)) {
		if (methodSet != null) {
			MethodCall mc = findMethodByName(m);
//			System.out.println("method call: " + m + " " + img.getImage());
			if (mc != null) {
				GraphNode prevNode = wrapper.getNodes().get(wrapper.getNodes().size() - 1);
				for (GraphNode g : wrapper.getNodes())
					if (g.getInitPC() == prevNode.getInitPC()) {
						g.setMethodCall((MethodCall)mc.clone());
					}
			}
//			System.out.println(wrapper.getNodes().get(wrapper.getNodes().size() - 1).getMethodCall());
		}

		super.visit(n);
	}
	
	/**
	 * @param methodName
	 * @return
	 */
	private MethodCall findMethodByName(String methodName) {
		if (methodName == null)
			return null;
		
		for (MethodCall mc : methodSet) {
			if (methodName.equalsIgnoreCase(mc.getMethodName()))
				return mc;
		}
		
		return null;
	}
	
	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.StatementExpression)
	 */
	@Override
	public void visit(StatementExpression n) {
		wrapper.setLoc(wrapper.getLoc() + 1);
		Node choice = n.f0.choice;
		ImageVisitor img;
		String varName = null;
		String assignment = null;

		if (choice instanceof NodeSequence) {
			NodeSequence ns = (NodeSequence)choice;

			img = new ImageVisitor();
			ns.elementAt(0).accept(img);
			varName = img.getImage();

			img = new ImageVisitor();
			ns.elementAt(1).accept(img);
			assignment = img.getImage();
		} else {
			// parsing caso per caso:
			System.out.println("StatementExpression choice is: " + choice.getClass().getName());

			if (choice instanceof PreIncrementExpression) {
				varName = ((PreIncrementExpression)n.f0.choice).f0.tokenImage;
				img = new ImageVisitor();
				((PreIncrementExpression)n.f0.choice).f1.accept(img);
				assignment = img.getImage();
			} else if (choice instanceof PreDecrementExpression) {
				varName = ((PreIncrementExpression)n.f0.choice).f0.tokenImage;
				img = new ImageVisitor();
				((PreDecrementExpression)n.f0.choice).f0.accept(img);
				assignment = img.getImage();
			} else if (choice instanceof Assignment) {
				img = new ImageVisitor();
				((Assignment)n.f0.choice).f0.accept(img);
				varName = img.getImage();

				img = new ImageVisitor();
				((Assignment)n.f0.choice).f1.accept(img);
				assignment = img.getImage();
				img = new ImageVisitor();
				((Assignment)n.f0.choice).f2.accept(img);
				assignment += img.getImage();
			} else if (choice instanceof PostfixExpression) {
				img = new ImageVisitor();
				((PostfixExpression)n.f0.choice).f0.accept(img);
				varName = img.getImage();
				System.out.println("varName: "+varName);

				img = new ImageVisitor();
				((PostfixExpression)n.f0.choice).f1.accept(img);
				assignment = img.getImage();
				System.out.println("Assignment: "+assignment);
				if (assignment.equals(" ++")) assignment = "="+varName + "+1";
				else if (assignment.equals(" --")) assignment = "="+varName + "-1";

			}
		}

		GraphNode node;
		if ((varName == null) || (assignment == null))
			node = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1);
		else
			node = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1, varName, assignment);
		wrapper.addNode(node);

		super.visit(n);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.IfStatement)
	 */
	@Override
	public void visit(IfStatement n) {
		wrapper.setLoc(wrapper.getLoc() + 1);

		LengthVisitor l = new LengthVisitor();
		n.accept(l);
		int ifLen = l.getLength();
		System.out.println("Lunghezza del if Statement: " + ifLen);

		ImageVisitor img = new ImageVisitor();
		n.f2.accept(img);
		String condition = img.getImage();
		System.out.println("Condizione dell'if Statement: " + condition);

		Condition cond1 = new Condition(condition);
		// condizione:
		GraphNode node1 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1, cond1);
		wrapper.getNodes().add(node1);


		Condition cond2 = new Condition(condition);
		// condizione negata:
		cond2.setNegated(true);
		int elseLen;
		// stabilire la lunghezza del ramo else:
		if (n.f5.node != null) {
			l = new LengthVisitor();
			n.f5.accept(l);
			elseLen = l.getLength();
			System.out.println("else: " + elseLen);
		} else {
			elseLen = 0;
		}
		GraphNode node2 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + ifLen - elseLen - 1, cond2);
		wrapper.getNodes().add(node2);

//		super.visit(n);
		n.f0.accept(this);
		n.f1.accept(this);
		n.f2.accept(this);
		n.f3.accept(this);
		n.f4.accept(this);

		// modificare l'ultimo nodo dello statement, in modo da fargli saltare il ramo else:
		if (n.f5.node != null) {
			int prevNodePC = wrapper.getNodes().get(wrapper.getNodes().size() - 1).getDestPC();
			System.out.println("edit " + wrapper.getNodes().get(wrapper.getNodes().size() - 1).getInitPC() + " from " + prevNodePC + " to " + (prevNodePC + elseLen));
			wrapper.getNodes().get(wrapper.getNodes().size() - 1).setDestPC(prevNodePC + elseLen);
		}

		// visitare il ramo else:
		n.f5.accept(this);

		// NOP:
		wrapper.setLoc(wrapper.getLoc() + 1);
		GraphNode node3 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1);
		wrapper.getNodes().add(node3);
	}

	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.WhileStatement)
	 */
	@Override
	public void visit(WhileStatement n) {
		wrapper.setLoc(wrapper.getLoc() + 1);

		LengthVisitor l= new LengthVisitor();
		n.accept(l);
		System.out.println("Lunghezza del while Statement: " + l.getLength());

		ImageVisitor img = new ImageVisitor();
		n.f2.accept(img);
		String condition = img.getImage();
		System.out.println("Condizione del while Statement: " + condition);

		// condizione:
		Condition cond1 = new Condition(condition);
		GraphNode node1 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1, cond1);
		wrapper.addNode(node1);

		System.out.println(wrapper.getLoc() + " " + (wrapper.getLoc() + l.getLength()));
		// condizione negata:
		Condition cond2 = new Condition(condition);
		cond2.setNegated(true);
		GraphNode node2 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + l.getLength(), cond2);
		wrapper.addNode(node2);

		int whileLoc = wrapper.getLoc();
		System.out.println(whileLoc);
		super.visit(n);

		// salto indietro:
		wrapper.setLoc(wrapper.getLoc() + 1);
//		System.out.println(whileLoc + " " + (wrapper.getLoc()));
//		GraphNode node3 = new GraphNode(whileLoc + l.getLength(), whileLoc);
		GraphNode node3 = new GraphNode(wrapper.getLoc(), whileLoc);
		wrapper.addNode(node3);
	}


	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.DoStatement)
	 */
	@Override
	public void visit(DoStatement n) {
		wrapper.setLoc(wrapper.getLoc());

		n.f1.accept(this);

		LengthVisitor l= new LengthVisitor();
		n.accept(l);
		System.out.println("Lunghezza del do Statement: " + l.getLength());

		ImageVisitor img = new ImageVisitor();
		n.f4.accept(img);
		String condition = img.getImage();
		System.out.println("Condizione del do Statement: " + condition);

		// condizione:
		Condition cond1 = new Condition(condition);
		GraphNode node1 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() -l.getLength()+2, cond1);
		wrapper.addNode(node1);

		System.out.println(wrapper.getLoc() + " " + (wrapper.getLoc() - l.getLength()));
		// condizione negata:
		Condition cond2 = new Condition(condition);
		cond2.setNegated(true);
		GraphNode node2 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + 2, cond2);
		wrapper.addNode(node2);

		int doLoc = wrapper.getLoc();
		System.out.println(doLoc);

		wrapper.setLoc(wrapper.getLoc()+1);

		//	super.visit(n);

		// salto indietro:
		//	wrapper.setLoc(wrapper.getLoc() + 1);
//		System.out.println(whileLoc + " " + (wrapper.getLoc()));
//		GraphNode node3 = new GraphNode(whileLoc + l.getLength(), whileLoc);
		//	GraphNode node3 = new GraphNode(wrapper.getLoc(), doLoc);
		//	wrapper.addNode(node3);
	}



	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.ForStatement)
	 */
	@Override
	public void visit(ForStatement n) {
		//wrapper.setLoc(wrapper.getLoc() + 3);
		Node primaParte = n.f2.node;
		Node secondaParte = n.f4.node;
		Node terzaParte = n.f6.node;
		LengthVisitor l= new LengthVisitor();

		/*
		 * parte 1 del for (1;2;3)
		 */
		if (primaParte instanceof ForInit) {
			ForInit fi = (ForInit)primaParte;
			if (fi.f0.choice instanceof LocalVariableDeclaration) visit((LocalVariableDeclaration)fi.f0.choice);
			else if (fi.f0.choice instanceof StatementExpressionList) visit((StatementExpressionList)fi.f0.choice);
		}

		/*
		 * parte 2 del for (1;2;3)
		 */
		if (secondaParte instanceof Expression) {
			Expression ex = (Expression)secondaParte;
			n.accept(l);
			System.out.println("Lunghezza del for Statement: " + l.getLength());

			ImageVisitor img = new ImageVisitor();
			ex.accept(img);
			String condition = img.getImage();
			System.out.println("Condizione del for Statement: " + condition);

			// condizione:
			Condition cond1 = new Condition(condition);
			GraphNode node1 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + 2, cond1);
			wrapper.addNode(node1);

			// condizione negata:
			Condition cond2 = new Condition(condition);
			cond2.setNegated(true);
			GraphNode node2 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + l.getLength(), cond2);
			wrapper.addNode(node2);
		}
		else {
			n.accept(l);
			System.out.println("Lunghezza del for Statement: " + l.getLength());

			String condition = " true";
			System.out.println("Condizione del for Statement: " + condition);

			// condizione:
			Condition cond1 = new Condition(condition);
			GraphNode node1 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + 2, cond1);
			wrapper.addNode(node1);

			// condizione negata:
			Condition cond2 = new Condition(condition);
			cond2.setNegated(true);
			GraphNode node2 = null;
			if (primaParte instanceof ForInit && !(secondaParte instanceof Expression) && !(terzaParte instanceof ForUpdate))
				node2 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + l.getLength(), cond2);
			else
				node2 = new GraphNode(wrapper.getLoc()+1, wrapper.getLoc() + l.getLength() + 1, cond2);
			wrapper.addNode(node2);
		}

		int forLoc = wrapper.getLoc();
		wrapper.setLoc(wrapper.getLoc() + 1);

		//super.visit(n);
		n.f8.accept(this);

		/*
		 * parte 3 del for (1;2;3)
		 */
		if (terzaParte instanceof ForUpdate) {
			ForUpdate fu = (ForUpdate)terzaParte;
			if (fu.f0 instanceof StatementExpressionList) visit((StatementExpressionList)fu.f0);
		}

		// salto indietro:
		wrapper.setLoc(wrapper.getLoc() + 1);
		System.out.println(wrapper.getLoc() + " " + (forLoc + l.getLength()));
		GraphNode node3 = null;
		if (secondaParte instanceof Expression && terzaParte instanceof ForUpdate)
//			node3 = new GraphNode(forLoc + l.getLength(), forLoc+1);
			node3 = new GraphNode(wrapper.getLoc(), forLoc+1);
		else if (!(primaParte instanceof ForInit) && !(secondaParte instanceof Expression) && !(terzaParte instanceof ForUpdate)) 
//			node3 = new GraphNode(forLoc + l.getLength()+1, forLoc+1);
			node3 = new GraphNode(wrapper.getLoc(), forLoc+1);
		else 
//			node3 = new GraphNode(forLoc + l.getLength(), forLoc+1);
			node3 = new GraphNode(wrapper.getLoc(), forLoc+1);
		wrapper.addNode(node3);

	}


	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.SwitchStatement)
	 */
	@Override
	public void visit(SwitchStatement n) {
		System.out.println("Trovato switch statement");
		wrapper.setLoc(wrapper.getLoc() + 1);

		LengthVisitor l= new LengthVisitor();
		n.accept(l);
		System.out.println("Lunghezza dello Switch Statement: " + l.getLength());

		ImageVisitor img = new ImageVisitor();
		n.f2.accept(img);
		String variabile = img.getImage();
		System.out.println("Condizione dello Switch Statement: " + variabile);


		//GraphNode node1 = new GraphNode(wrapper.getLoc(), wrapper.getLoc() + 1);
		//wrapper.addNode(node1);

		System.out.println(wrapper.getLoc() + " " + (wrapper.getLoc() + l.getLength()));
		// condizione negata:
		int switchLoc = wrapper.getLoc();
		System.out.println(switchLoc);
		//super.visit(n);
		NodeListOptional nlo = (NodeListOptional) (n.f5);
		int i = 0;
		String condTotale = "";
		for (Node node : nlo.nodes) {
			if (node instanceof NodeSequence) {
				NodeSequence ns = (NodeSequence)node;
				
				for (Node node2 : ns.nodes) {
					if (node2 instanceof SwitchLabel) {
						SwitchLabel sl = (SwitchLabel)node2;
						ImageVisitor img2 = new ImageVisitor();
						sl.accept(img2);
						String condition = img2.getImage();
						System.out.println("Switch label: " + condition);

						// condizione:
						Condition cond1 = new Condition(variabile+"="+condition.substring(5,condition.length()-1));
						if (condition.indexOf("default")>-1) {
							cond1 = new Condition("!("+condTotale+" )");
						}
						else condTotale+=(i>0?" |":"")+variabile+"="+condition.substring(5,condition.length()-1);
						
						LengthVisitor l2= new LengthVisitor();
						ns.accept(l2);
						GraphNode node3 = new GraphNode(wrapper.getLoc(), wrapper.getLoc()+1+i, cond1);
						wrapper.addNode(node3);
						i+=l2.getLength();
						System.out.println("L2="+l2.getLength());
					}
				}
			}
			this.breakPoint=l.getLength()+2;
		}
		//wrapper.setLoc(wrapper.getLoc()+1);
	    nlo.accept(this);
	}
	
	
	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.BreakStatement)
	 */	
	@Override
	 public void visit(BreakStatement n) {
		 System.out.println("Freeena!");
		 GraphNode node = new GraphNode(wrapper.getLoc()+1, breakPoint);
			wrapper.addNode(node);
			wrapper.setLoc(wrapper.getLoc()+1);
	 }



	/* (non-Javadoc)
	 * @see EDU.purdue.jtb.visitor.DepthFirstVisitor#visit(EDU.purdue.jtb.syntaxtree.ReturnStatement)
	 */
	@Override
	public void visit(ReturnStatement n) {
		hasReturn = true;

		wrapper.setLoc(wrapper.getLoc() + 1);
		wrapper.getNumTermIstr().addElement(new Integer(wrapper.getLoc()));
		GraphNode node = new GraphNode(wrapper.getLoc(), wrapper.getLoc());
		wrapper.addNode(node);

		super.visit(n);
	}

	/**
	 * @param n
	 * @return
	 */
	private static String typeVariable(NodeChoice n){
		String tipo = "";
		switch (n.which) {
		case 0: 
			tipo = "boolean"; 
			break;
		case 1: 
			tipo = "char";
			break;
		case 2: 
			tipo = "byte";
			break;
		case 3: 
			tipo = "short";
			break;
		case 4: 
			tipo = "int";
			break;
		case 5: 
			tipo = "long";
			break;
		case 6: 
			tipo = "float";
			break;
		case 7: 
			tipo = "double";
			break;
		}
		return tipo;
	}
	
	/**
	 * Rimuove gli spazi ' ' da una stringa.
	 * @param s
	 * @return
	 */
	private static String stripSpace(String s) {
		if (s == null)
			return s;
		else
			return s.replace(" ", "");
	}
}
